/**
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

// RUN: %hermesc -O0 -dump-ir %s | %FileCheckOrRegen --match-full-lines %s

function f1(t) {
    var {...a} = t;
    return a;
}

function f2(t) {
    var {a, b, ...rest} = t;
    return rest;
}

function f3(t) {
    var a, rest;
    ({a, ...rest} = t);
}

function f4(o, t) {
    var a;
    ({a, ...o.rest} = t);
}

function f5(o) {
    var a, rest;
    ({a, a,  ...rest} = o);
}

// Auto-generated content below. Please do not modify manually.

// CHECK:function global#0()#1
// CHECK-NEXT:globals = [f1, f2, f3, f4, f5]
// CHECK-NEXT:S{global#0()#1} = []
// CHECK-NEXT:%BB0:
// CHECK-NEXT:  %0 = CreateScopeInst %S{global#0()#1}
// CHECK-NEXT:  %1 = CreateFunctionInst %f1#0#1()#2, %0
// CHECK-NEXT:  %2 = StorePropertyInst %1 : closure, globalObject : object, "f1" : string
// CHECK-NEXT:  %3 = CreateFunctionInst %f2#0#1()#3, %0
// CHECK-NEXT:  %4 = StorePropertyInst %3 : closure, globalObject : object, "f2" : string
// CHECK-NEXT:  %5 = CreateFunctionInst %f3#0#1()#4, %0
// CHECK-NEXT:  %6 = StorePropertyInst %5 : closure, globalObject : object, "f3" : string
// CHECK-NEXT:  %7 = CreateFunctionInst %f4#0#1()#5, %0
// CHECK-NEXT:  %8 = StorePropertyInst %7 : closure, globalObject : object, "f4" : string
// CHECK-NEXT:  %9 = CreateFunctionInst %f5#0#1()#6, %0
// CHECK-NEXT:  %10 = StorePropertyInst %9 : closure, globalObject : object, "f5" : string
// CHECK-NEXT:  %11 = AllocStackInst $?anon_0_ret
// CHECK-NEXT:  %12 = StoreStackInst undefined : undefined, %11
// CHECK-NEXT:  %13 = LoadStackInst %11
// CHECK-NEXT:  %14 = ReturnInst %13
// CHECK-NEXT:function_end

// CHECK:function f1#0#1(t)#2
// CHECK-NEXT:S{f1#0#1()#2} = [t#2, a#2]
// CHECK-NEXT:%BB0:
// CHECK-NEXT:  %0 = CreateScopeInst %S{f1#0#1()#2}
// CHECK-NEXT:  %1 = StoreFrameInst %t, [t#2], %0
// CHECK-NEXT:  %2 = StoreFrameInst undefined : undefined, [a#2], %0
// CHECK-NEXT:  %3 = LoadFrameInst [t#2], %0
// CHECK-NEXT:  %4 = BinaryOperatorInst '==', %3, null : null
// CHECK-NEXT:  %5 = CondBranchInst %4, %BB1, %BB2
// CHECK-NEXT:%BB1:
// CHECK-NEXT:  %6 = CallBuiltinInst [HermesBuiltin.throwTypeError] : number, undefined : undefined, undefined : undefined, %3, "Cannot destructure 'undefined' or 'null'." : string
// CHECK-NEXT:  %7 = ThrowInst undefined : undefined
// CHECK-NEXT:%BB2:
// CHECK-NEXT:  %8 = AllocObjectInst 0 : number, empty
// CHECK-NEXT:  %9 = CallBuiltinInst [HermesBuiltin.copyDataProperties] : number, undefined : undefined, undefined : undefined, %8 : object, %3, undefined : undefined
// CHECK-NEXT:  %10 = StoreFrameInst %9, [a#2], %0
// CHECK-NEXT:  %11 = LoadFrameInst [a#2], %0
// CHECK-NEXT:  %12 = ReturnInst %11
// CHECK-NEXT:%BB3:
// CHECK-NEXT:  %13 = ReturnInst undefined : undefined
// CHECK-NEXT:function_end

// CHECK:function f2#0#1(t)#3
// CHECK-NEXT:S{f2#0#1()#3} = [t#3, a#3, b#3, rest#3]
// CHECK-NEXT:%BB0:
// CHECK-NEXT:  %0 = CreateScopeInst %S{f2#0#1()#3}
// CHECK-NEXT:  %1 = StoreFrameInst %t, [t#3], %0
// CHECK-NEXT:  %2 = StoreFrameInst undefined : undefined, [a#3], %0
// CHECK-NEXT:  %3 = StoreFrameInst undefined : undefined, [b#3], %0
// CHECK-NEXT:  %4 = StoreFrameInst undefined : undefined, [rest#3], %0
// CHECK-NEXT:  %5 = LoadFrameInst [t#3], %0
// CHECK-NEXT:  %6 = LoadPropertyInst %5, "a" : string
// CHECK-NEXT:  %7 = StoreFrameInst %6, [a#3], %0
// CHECK-NEXT:  %8 = LoadPropertyInst %5, "b" : string
// CHECK-NEXT:  %9 = StoreFrameInst %8, [b#3], %0
// CHECK-NEXT:  %10 = AllocObjectInst 2 : number, null : null
// CHECK-NEXT:  %11 = StoreNewOwnPropertyInst 0 : number, %10 : object, "a" : string, true : boolean
// CHECK-NEXT:  %12 = StoreNewOwnPropertyInst 0 : number, %10 : object, "b" : string, true : boolean
// CHECK-NEXT:  %13 = AllocObjectInst 0 : number, empty
// CHECK-NEXT:  %14 = CallBuiltinInst [HermesBuiltin.copyDataProperties] : number, undefined : undefined, undefined : undefined, %13 : object, %5, %10 : object
// CHECK-NEXT:  %15 = StoreFrameInst %14, [rest#3], %0
// CHECK-NEXT:  %16 = LoadFrameInst [rest#3], %0
// CHECK-NEXT:  %17 = ReturnInst %16
// CHECK-NEXT:%BB1:
// CHECK-NEXT:  %18 = ReturnInst undefined : undefined
// CHECK-NEXT:function_end

// CHECK:function f3#0#1(t)#4
// CHECK-NEXT:S{f3#0#1()#4} = [t#4, a#4, rest#4]
// CHECK-NEXT:%BB0:
// CHECK-NEXT:  %0 = CreateScopeInst %S{f3#0#1()#4}
// CHECK-NEXT:  %1 = StoreFrameInst %t, [t#4], %0
// CHECK-NEXT:  %2 = StoreFrameInst undefined : undefined, [a#4], %0
// CHECK-NEXT:  %3 = StoreFrameInst undefined : undefined, [rest#4], %0
// CHECK-NEXT:  %4 = LoadFrameInst [t#4], %0
// CHECK-NEXT:  %5 = LoadPropertyInst %4, "a" : string
// CHECK-NEXT:  %6 = StoreFrameInst %5, [a#4], %0
// CHECK-NEXT:  %7 = AllocObjectInst 1 : number, null : null
// CHECK-NEXT:  %8 = StoreNewOwnPropertyInst 0 : number, %7 : object, "a" : string, true : boolean
// CHECK-NEXT:  %9 = AllocObjectInst 0 : number, empty
// CHECK-NEXT:  %10 = CallBuiltinInst [HermesBuiltin.copyDataProperties] : number, undefined : undefined, undefined : undefined, %9 : object, %4, %7 : object
// CHECK-NEXT:  %11 = StoreFrameInst %10, [rest#4], %0
// CHECK-NEXT:  %12 = ReturnInst undefined : undefined
// CHECK-NEXT:function_end

// CHECK:function f4#0#1(o, t)#5
// CHECK-NEXT:S{f4#0#1()#5} = [o#5, t#5, a#5]
// CHECK-NEXT:%BB0:
// CHECK-NEXT:  %0 = CreateScopeInst %S{f4#0#1()#5}
// CHECK-NEXT:  %1 = StoreFrameInst %o, [o#5], %0
// CHECK-NEXT:  %2 = StoreFrameInst %t, [t#5], %0
// CHECK-NEXT:  %3 = StoreFrameInst undefined : undefined, [a#5], %0
// CHECK-NEXT:  %4 = LoadFrameInst [t#5], %0
// CHECK-NEXT:  %5 = LoadPropertyInst %4, "a" : string
// CHECK-NEXT:  %6 = StoreFrameInst %5, [a#5], %0
// CHECK-NEXT:  %7 = LoadFrameInst [o#5], %0
// CHECK-NEXT:  %8 = AllocObjectInst 1 : number, null : null
// CHECK-NEXT:  %9 = StoreNewOwnPropertyInst 0 : number, %8 : object, "a" : string, true : boolean
// CHECK-NEXT:  %10 = AllocObjectInst 0 : number, empty
// CHECK-NEXT:  %11 = CallBuiltinInst [HermesBuiltin.copyDataProperties] : number, undefined : undefined, undefined : undefined, %10 : object, %4, %8 : object
// CHECK-NEXT:  %12 = StorePropertyInst %11, %7, "rest" : string
// CHECK-NEXT:  %13 = ReturnInst undefined : undefined
// CHECK-NEXT:function_end

// CHECK:function f5#0#1(o)#6
// CHECK-NEXT:S{f5#0#1()#6} = [o#6, a#6, rest#6]
// CHECK-NEXT:%BB0:
// CHECK-NEXT:  %0 = CreateScopeInst %S{f5#0#1()#6}
// CHECK-NEXT:  %1 = StoreFrameInst %o, [o#6], %0
// CHECK-NEXT:  %2 = StoreFrameInst undefined : undefined, [a#6], %0
// CHECK-NEXT:  %3 = StoreFrameInst undefined : undefined, [rest#6], %0
// CHECK-NEXT:  %4 = LoadFrameInst [o#6], %0
// CHECK-NEXT:  %5 = LoadPropertyInst %4, "a" : string
// CHECK-NEXT:  %6 = StoreFrameInst %5, [a#6], %0
// CHECK-NEXT:  %7 = LoadPropertyInst %4, "a" : string
// CHECK-NEXT:  %8 = StoreFrameInst %7, [a#6], %0
// CHECK-NEXT:  %9 = AllocObjectInst 1 : number, null : null
// CHECK-NEXT:  %10 = StoreNewOwnPropertyInst 0 : number, %9 : object, "a" : string, true : boolean
// CHECK-NEXT:  %11 = AllocObjectInst 0 : number, empty
// CHECK-NEXT:  %12 = CallBuiltinInst [HermesBuiltin.copyDataProperties] : number, undefined : undefined, undefined : undefined, %11 : object, %4, %9 : object
// CHECK-NEXT:  %13 = StoreFrameInst %12, [rest#6], %0
// CHECK-NEXT:  %14 = ReturnInst undefined : undefined
// CHECK-NEXT:function_end
