/*
 * Copyright (c) Meta Platforms, Inc. and affiliates.
 *
 * This source code is licensed under the MIT license found in the
 * LICENSE file in the root directory of this source tree.
 */

#include "hermes/VM/NativeState.h"

#include "TestHelpers.h"
#include "gtest/gtest.h"

using namespace hermes::vm;

namespace {

using NativeStateTest = RuntimeTestFixture;

void inc(void *context) {
  ++*(int *)context;
}

TEST_F(NativeStateTest, NativeStateFinalizerRunsOnce) {
  int counter = 0;
  {
    GCScope scope{runtime, "NativeStateTest"};
    (void)runtime.makeHandle(NativeState::create(runtime, &counter, inc));
    runtime.collect("test");
    // should not have been finalized yet
    EXPECT_EQ(0, counter);
  }
  // should finalize once
  runtime.collect("test");
  EXPECT_EQ(1, counter);
  runtime.collect("test");
  runtime.collect("test");
  EXPECT_EQ(1, counter);
}

} // namespace
